/*************************************************************************
* Demo sketch for MicroLCD library
* Distributed under GPL v2.0
* Copyright (c) 2013-2014 Stanley Huang <stanleyhuangyc@gmail.com>
* All rights reserved.
* For more information, please visit http://arduinodev.com
*************************************************************************/

#include <Arduino.h>
#include <Wire.h>
#include <MicroLCD.h>

//LCD_SH1106 lcd; /* for SH1106 OLED module */
LCD_SSD1306 lcd; /* for SSD1306 OLED module */

const PROGMEM uint8_t smile[48 * 48 / 8] = {
0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xC0,0xE0,0xF0,0xF8,0xF8,0xFC,0xFC,0xFE,0xFE,0x7E,0x7F,0x7F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x7F,0x7F,0x7E,0xFE,0xFE,0xFC,0xFC,0xF8,0xF8,0xF0,0xE0,0xC0,0x80,0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0xC0,0xF0,0xFC,0xFE,0xFF,0xFF,0xFF,0x3F,0x1F,0x0F,0x07,0x03,0x01,0x00,0x80,0x80,0x80,0x80,0x80,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x80,0x80,0x80,0x80,0x80,0x00,0x01,0x03,0x07,0x0F,0x1F,0x3F,0xFF,0xFF,0xFF,0xFE,0xFC,0xF0,0xC0,0x00,
0xFE,0xFF,0xFF,0xFF,0xFF,0xFF,0x07,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x06,0x1F,0x1F,0x1F,0x3F,0x1F,0x1F,0x02,0x00,0x00,0x00,0x00,0x06,0x1F,0x1F,0x1F,0x3F,0x1F,0x1F,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x07,0xFF,0xFF,0xFF,0xFF,0xFF,0xFE,
0x7F,0xFF,0xFF,0xFF,0xFF,0xFF,0xE0,0x00,0x00,0x30,0xF8,0xF8,0xF8,0xF8,0xE0,0xC0,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xC0,0xE0,0xF8,0xF8,0xFC,0xF8,0x30,0x00,0x00,0xE0,0xFF,0xFF,0xFF,0xFF,0xFF,0x7F,
0x00,0x03,0x0F,0x3F,0x7F,0xFF,0xFF,0xFF,0xFC,0xF8,0xF0,0xE1,0xC7,0x87,0x0F,0x1F,0x3F,0x3F,0x3E,0x7E,0x7C,0x7C,0x7C,0x78,0x78,0x7C,0x7C,0x7C,0x7E,0x3E,0x3F,0x3F,0x1F,0x0F,0x87,0xC7,0xE1,0xF0,0xF8,0xFC,0xFF,0xFF,0xFF,0x7F,0x3F,0x0F,0x03,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x03,0x07,0x0F,0x1F,0x1F,0x3F,0x3F,0x7F,0x7F,0x7E,0xFE,0xFE,0xFC,0xFC,0xFC,0xFC,0xFC,0xFC,0xFC,0xFC,0xFC,0xFC,0xFE,0xFE,0x7E,0x7F,0x7F,0x3F,0x3F,0x1F,0x1F,0x0F,0x07,0x03,0x01,0x00,0x00,0x00,0x00,0x00,0x00,
};

const PROGMEM uint8_t tick[16 * 16 / 8] =
{0x00,0x80,0xC0,0xE0,0xC0,0x80,0x00,0x80,0xC0,0xE0,0xF0,0xF8,0xFC,0x78,0x30,0x00,0x00,0x01,0x03,0x07,0x0F,0x1F,0x1F,0x1F,0x0F,0x07,0x03,0x01,0x00,0x00,0x00,0x00};

const PROGMEM uint8_t cross[16 * 16 / 8] =
{0x00,0x0C,0x1C,0x3C,0x78,0xF0,0xE0,0xC0,0xE0,0xF0,0x78,0x3C,0x1C,0x0C,0x00,0x00,0x00,0x30,0x38,0x3C,0x1E,0x0F,0x07,0x03,0x07,0x0F,0x1E,0x3C,0x38,0x30,0x00,0x00};


#define Sampling_Time 5
#define Num_Samples 600
#define Peak_Threshold_Factor 85
#define Minimum_Range 500
#define Minimum_Peak_Separation 75  // 75*5=375 ms
#define DC_Added 10;
#define Moving_Average_Num 5
#define Samples_to_Display 400
#define Display_Sampling 5

int ADC_Samples[Num_Samples], Index1, Index2, Index3, Peak1, Peak2, Peak3;
long Pulse_Rate, Temp1=1L, Pulse_Time1, Pulse_Time2;
int Peak_Magnitude, Peak_Threshold, Minima, Range;

void setup()
{
	lcd.begin();
lcd.clear();
 Serial.begin(9600);
}

void loop()
{
	
	
	delay(100);
        
        Read_ADC_Samples();
  Serial.println("Sample Read Finished ");
  Remove_DC();
  Serial.println("DC component subtracted ");
  Scale_Data();
  Serial.println("Data scaled ");
  
  if (Range > Minimum_Range){  // ADC range is > 70, otherwise increase gain
    Filter_Data();
    Serial.println("Data Filtered ");
    Compute_Pulse_Rate();
    Serial.println("Pulse rate computed ");
    draw_PPG();
  }
  else{
    lcd.clear();
    lcd.setFontSize(FONT_SIZE_MEDIUM);
    lcd.setCursor(15, 3);
    lcd.println("Sensor Error");
	
  }
}

// Read ADC samples at a predefined interval
 void Read_ADC_Samples(){
   for (int i = 0; i < Num_Samples; i++){
      ADC_Samples[i] = analogRead(A0);
      delay(Sampling_Time);  
   }  
 }

 void Remove_DC(){
   Find_Minima(0);
   Serial.print("Minima = ");
   Serial.println(Minima);
   // Subtract DC (minima) 
   for (int i = 0; i < Num_Samples; i++){
     ADC_Samples[i] = ADC_Samples[i] - Minima;
   }
   Minima = 0;  // New minima is zero  
 }  // Remove_DC

void Find_Minima(int Num){
  Minima = 1024;
  for (int m = Num; m < Num_Samples-Num; m++){
      if(Minima > ADC_Samples[m]){
         Minima = ADC_Samples[m];
      }
  }
}  // Find_Minima  

void Scale_Data(){
  // Find peak value
  Find_Peak(0);
  Serial.print("Peak = ");
  Serial.println(Peak_Magnitude);
  Serial.print("Minima = ");
  Serial.println(Minima);
  Range = Peak_Magnitude - Minima;
  Serial.print("Range = ");
  Serial.println(Range);
  // Sclae from 1 to 1023 
  for (int i = 0; i < Num_Samples; i++){
     ADC_Samples[i] = map(ADC_Samples[i], 0, Range, 1, 1023);
     
  }
  Find_Peak(0);
  Find_Minima(0);
  Serial.print("Scaled Peak = ");
  Serial.println(Peak_Magnitude);
  Serial.print("Scaled Minima = ");
  Serial.println(Minima);
  
} // Scale_Data

void Find_Peak(int Num){
  Peak_Magnitude = 0;
  for (int m = Num; m < Num_Samples-Num; m++){
      if(Peak_Magnitude < ADC_Samples[m]){
        Peak_Magnitude = ADC_Samples[m];
     }
  }
} // Find_Peak

void Filter_Data(){
  int Num_Points = 2*Moving_Average_Num+1;
  for (int i = Moving_Average_Num; i < Num_Samples-Moving_Average_Num; i++){
    int Sum_Points = 0;
    for(int k =0; k < Num_Points; k++){   
      Sum_Points = Sum_Points + ADC_Samples[i-Moving_Average_Num+k]; 
    }    
    ADC_Samples[i] = Sum_Points/Num_Points; 
  } 
}  // Filter_Date


void Compute_Pulse_Rate(){
  // Detect Peak magnitude and minima
  Find_Peak(Moving_Average_Num);
  Find_Minima(Moving_Average_Num);
  Range = Peak_Magnitude - Minima;
  Peak_Threshold = Peak_Magnitude*Peak_Threshold_Factor;
  Peak_Threshold = Peak_Threshold/100;
 
  // Now detect three peaks 
  Peak1 = 0;
  Peak2 = 0;
  Peak3 = 0;
  Index1 = 0;
  Index2 = 0;
  Index3 = 0;
  // Find first peak
  for (int j = Moving_Average_Num; j < Num_Samples-Moving_Average_Num; j++){
      if(ADC_Samples[j] >= ADC_Samples[j-1] && ADC_Samples[j] > ADC_Samples[j+1] && 
         ADC_Samples[j] > Peak_Threshold && Peak1 == 0){
        Peak1 = ADC_Samples[j];
        Index1 = j; 
      }
      
      // Search for second peak which is at least 10 sample time far
      if(Peak1 > 0 && j > (Index1+Minimum_Peak_Separation) && Peak2 == 0){
         if(ADC_Samples[j] >= ADC_Samples[j-1] && ADC_Samples[j] > ADC_Samples[j+1] && 
         ADC_Samples[j] > Peak_Threshold){
         Peak2 = ADC_Samples[j];
         Index2 = j; 
         } 
      } // Peak1 > 0
      
      // Search for the third peak which is at least 10 sample time far
      if(Peak2 > 0 && j > (Index2+Minimum_Peak_Separation) && Peak3 == 0){
         if(ADC_Samples[j] >= ADC_Samples[j-1] && ADC_Samples[j] > ADC_Samples[j+1] && 
         ADC_Samples[j] > Peak_Threshold){
            Peak3 = ADC_Samples[j];
            Index3 = j; 
         } 
      } // Peak2 > 0
    
  }
  Serial.print("Index1 = ");
  Serial.println(Index1);
  Serial.print("Index2 = ");
  Serial.println(Index2);
  Serial.print("Index3 = ");
  Serial.println(Index3);
  
  
  Pulse_Time1 = (Index2-Index1)*Sampling_Time; // In milliseconds
  Pulse_Time2 = (Index3-Index2)*Sampling_Time;
  
  if(Pulse_Time1 > 0 && Pulse_Time1 > 0){
    Pulse_Rate = 2*60000/(Pulse_Time1+Pulse_Time2); // In BPM
  }
  Serial.print("Pulse Rate (BPM) = ");
  Serial.println(Pulse_Rate);
}  // Compute_Pulse_Rate


void draw_PPG(){
        lcd.clear();
        lcd.setFontSize(FONT_SIZE_SMALL);
        lcd.setCursor(2, 1);	
        lcd.print("Easy Pulse Plugin");
	lcd.setFontSize(FONT_SIZE_MEDIUM);
        lcd.setCursor(24, 4);	
        lcd.print(Pulse_Rate);
        lcd.print(" BPM");
        
	
 }

